@@ -2,12 +2,12 @@
 # docs: file://./../../../../docs/source/commands.rst
 import logging
 import sys
-from typing import Any  # pylint: disable=W
+from typing import Any
 
 import click
 
 from ....env_mgr.tfenv import TFEnvManager
-from ....util import DOC_SITE
+from ....utils import DOC_SITE
 from ... import options
 
 LOGGER = logging.getLogger(__name__.replace("._", "."))
@@ -18,13 +18,12 @@ LOGGER = logging.getLogger(__name__.replace("._", "."))
 @options.debug
 @options.no_color
 @options.verbose
-def install(version, **_):
-    # type: (str, Any) -> None
+def install(version: str, **_: Any) -> None:
     """Install the specified <version> of Terraform (e.g. 0.12.0).
 
     If no version is specified, Runway will attempt to find and read a
-    ".terraform-version" file in the current directory. If this file doesn't
-    exist, nothing will be installed.
+    ".terraform-version" file in the current directory.
+    If this file doesn't exist, nothing will be installed.
 
     """
     try:
