@@ -1,19 +1,20 @@
 """Runway CLI logging setup."""
 import logging
 import os
+from typing import Any, Dict
 
 import coloredlogs
 
 from runway import LogLevels
 
-from ..util import cached_property
+from ..compat import cached_property
 
 # COLOR_FORMAT = "%(levelname)s:%(name)s:\033[%(color)sm%(message)s\033[39m"
 LOGGER = logging.getLogger("runway")
 
 LOG_FORMAT = "[%(programname)s] %(message)s"
 LOG_FORMAT_VERBOSE = logging.BASIC_FORMAT
-LOG_FIELD_STYLES = {
+LOG_FIELD_STYLES: Dict[str, Dict[str, Any]] = {
     "asctime": {},
     "hostname": {},
     "levelname": {},
@@ -22,7 +23,7 @@ LOG_FIELD_STYLES = {
     "prefix": {},
     "programname": {},
 }
-LOG_LEVEL_STYLES = {
+LOG_LEVEL_STYLES: Dict[str, Dict[str, Any]] = {
     "critical": {"color": "red", "bold": True},
     "debug": {"color": "green"},
     "error": {"color": "red"},
@@ -35,7 +36,7 @@ LOG_LEVEL_STYLES = {
 }
 
 
-class LogSettings(object):
+class LogSettings:
     """CLI log settings."""
 
     ENV = {
@@ -44,13 +45,15 @@ class LogSettings(object):
         "level_styles": os.getenv("RUNWAY_LOG_LEVEL_STYLES"),
     }
 
-    def __init__(self, debug=0, no_color=False, verbose=False):
+    def __init__(
+        self, *, debug: int = 0, no_color: bool = False, verbose: bool = False
+    ):
         """Instantiate class.
 
         Args:
-            debug (int): Debug level.
-            no_color (bool): Disable color in Runway's logs.
-            verbose (bool): Whether to display verbose logs.
+            debug: Debug level.
+            no_color: Disable color in Runway's logs.
+            verbose: Whether to display verbose logs.
 
         """
         self.debug = debug
@@ -58,13 +61,8 @@ class LogSettings(object):
         self.verbose = verbose
 
     @property
-    def coloredlogs(self):
-        """Return settings for coloredlogs.
-
-        Returns:
-            Dict[str, Any]
-
-        """
+    def coloredlogs(self) -> Dict[str, Any]:
+        """Return settings for coloredlogs."""
         return {
             "fmt": self.fmt,
             "field_styles": self.field_styles,
@@ -72,67 +70,62 @@ class LogSettings(object):
         }
 
     @cached_property
-    def fmt(self):
+    def fmt(self) -> str:
         """Return log record format.
 
         If "RUNWAY_LOG_FORMAT" exists in the environment, it will be used.
 
-        Returns:
-            str
-
         """
-        if self.ENV["fmt"]:
-            return self.ENV["fmt"]
+        fmt = self.ENV["fmt"]
+        if isinstance(fmt, str):
+            return fmt
         if self.debug or self.no_color or self.verbose:
             return LOG_FORMAT_VERBOSE
         return LOG_FORMAT
 
     @cached_property
-    def field_styles(self):
+    def field_styles(self) -> Dict[str, Any]:
         """Return log field styles.
 
         If "RUNWAY_LOG_FIELD_STYLES" exists in the environment, it will be
         used to update the Runway LOG_FIELD_STYLES.
 
-        Returns:
-            Dict[str, Any]
-
         """
         if self.no_color:
             return {}
 
         result = LOG_FIELD_STYLES.copy()
         if self.ENV["field_styles"]:
-            result.update(coloredlogs.parse_encoded_styles(self.ENV["field_styles"]))
+            result.update(
+                coloredlogs.parse_encoded_styles(  # type: ignore
+                    self.ENV["field_styles"]
+                )
+            )
         return result
 
     @cached_property
-    def level_styles(self):
+    def level_styles(self) -> Dict[str, Any]:
         """Return log level styles.
 
         If "RUNWAY_LOG_LEVEL_STYLES" exists in the environment, it will be
         used to update the Runway LOG_LEVEL_STYLES.
 
-        Returns:
-            Dict[str, Any]
-
         """
         if self.no_color:
             return {}
 
         result = LOG_LEVEL_STYLES.copy()
         if self.ENV["level_styles"]:
-            result.update(coloredlogs.parse_encoded_styles(self.ENV["level_styles"]))
+            result.update(
+                coloredlogs.parse_encoded_styles(  # type: ignore
+                    self.ENV["level_styles"]
+                )
+            )
         return result
 
     @cached_property
-    def log_level(self):
-        """Return log level to use.
-
-        Returns:
-            LogLevel
-
-        """
+    def log_level(self) -> LogLevels:
+        """Return log level to use."""
         if self.debug:
             return LogLevels.DEBUG
         if self.verbose:
@@ -140,19 +133,18 @@ class LogSettings(object):
         return LogLevels.INFO
 
 
-# TODO implement propper keyword-only args when dropping python 2
-def setup_logging(*_, **kwargs):
+def setup_logging(
+    *, debug: int = 0, no_color: bool = False, verbose: bool = False
+) -> None:
     """Configure log settings for Runway CLI.
 
     Keyword Args:
-        debug (int): Debug level (0-2).
+        debug: Debug level (0-2).
+        no_color: Whether to use colorized logs.
+        verbose: Use verbose logging.
 
     """
-    settings = LogSettings(
-        debug=kwargs.pop("debug", 0),
-        no_color=kwargs.pop("no_color", False),
-        verbose=kwargs.pop("verbose", False),
-    )
+    settings = LogSettings(debug=debug, no_color=no_color, verbose=verbose)
 
     coloredlogs.install(settings.log_level, logger=LOGGER, **settings.coloredlogs)
     LOGGER.debug("runway log level: %s", LOGGER.getEffectiveLevel())
@@ -161,7 +153,7 @@ def setup_logging(*_, **kwargs):
         coloredlogs.install(
             settings.log_level,
             logger=logging.getLogger("botocore"),
-            **settings.coloredlogs
+            **settings.coloredlogs,
         )
         LOGGER.debug("set dependency log level to debug")
     LOGGER.debug("initalized logging for Runway")
