@@ -1,16 +1,14 @@
 #!/usr/bin/env python
 """Module with static website supporting infrastructure."""
-from __future__ import print_function
-
 import logging
 
 import awacs.s3
 from awacs.aws import Allow, AWSPrincipal, Policy, Statement
 from troposphere import AccountId, Join, Output, cognito, s3
 
-from runway.cfngin.blueprints.base import Blueprint
-from runway.hooks.staticsite.auth_at_edge.client_updater import get_redirect_uris
-from runway.module.staticsite import add_url_scheme
+from ...cfngin.blueprints.base import Blueprint
+from ...cfngin.hooks.staticsite.auth_at_edge.client_updater import get_redirect_uris
+from ...module.staticsite.utils import add_url_scheme
 
 LOGGER = logging.getLogger(__name__)
 
@@ -68,10 +66,9 @@ class Dependencies(Blueprint):
         },
     }
 
-    def create_template(self):
+    def create_template(self) -> None:
         """Create template (main function called by Stacker)."""
         template = self.template
-        variables = self.get_variables()
         template.set_version("2010-09-09")
         template.set_description("Static Website - Dependencies")
 
@@ -134,24 +131,27 @@ class Dependencies(Blueprint):
             )
         )
 
-        if variables["AuthAtEdge"]:
+        if self.variables["AuthAtEdge"]:
             userpool_client_params = {
                 "AllowedOAuthFlows": ["code"],
-                "AllowedOAuthScopes": variables["OAuthScopes"],
+                "AllowedOAuthScopes": self.variables["OAuthScopes"],
             }
-            if variables["Aliases"]:
+            if self.variables["Aliases"]:
                 userpool_client_params["AllowedOAuthFlowsUserPoolClient"] = True
-                userpool_client_params["SupportedIdentityProviders"] = variables[
+                userpool_client_params["SupportedIdentityProviders"] = self.variables[
                     "SupportedIdentityProviders"
                 ]
 
-                redirect_domains = [add_url_scheme(x) for x in variables["Aliases"]] + [
-                    add_url_scheme(x) for x in variables["AdditionalRedirectDomains"]
+                redirect_domains = [
+                    add_url_scheme(x) for x in self.variables["Aliases"]
+                ] + [
+                    add_url_scheme(x)
+                    for x in self.variables["AdditionalRedirectDomains"]
                 ]
                 redirect_uris = get_redirect_uris(
                     redirect_domains,
-                    variables["RedirectPathSignIn"],
-                    variables["RedirectPathSignOut"],
+                    self.variables["RedirectPathSignIn"],
+                    self.variables["RedirectPathSignOut"],
                 )
                 userpool_client_params["CallbackURLs"] = redirect_uris["sign_in"]
                 userpool_client_params["LogoutURLs"] = redirect_uris["sign_out"]
@@ -160,7 +160,7 @@ class Dependencies(Blueprint):
                     "aae_callback_url_retriever"
                 ]["callback_urls"]
 
-            if variables["CreateUserPool"]:
+            if self.variables["CreateUserPool"]:
                 user_pool = template.add_resource(
                     cognito.UserPool("AuthAtEdgeUserPool")
                 )
@@ -196,6 +196,8 @@ class Dependencies(Blueprint):
 # Helper section to enable easy blueprint -> template generation
 # (just run `python <thisfile>` to output the json)
 if __name__ == "__main__":
-    from runway.cfngin.context import Context
+    from runway.context import CfnginContext
 
-    print(Dependencies("test", Context({"namespace": "test"}), None).to_json())
+    print(
+        Dependencies("test", CfnginContext(parameters={"namespace": "test"})).to_json()
+    )
