@@ -1,5 +1,6 @@
 """CFNgin info action."""
 import logging
+from typing import Any
 
 from .. import exceptions
 from .base import BaseAction
@@ -7,7 +8,7 @@ from .base import BaseAction
 LOGGER = logging.getLogger(__name__)
 
 
-class Action(BaseAction):  # pylint: disable=abstract-method
+class Action(BaseAction):
     """Get information on CloudFormation stacks.
 
     Displays the outputs for the set of CloudFormation stacks.
@@ -16,20 +17,23 @@ class Action(BaseAction):  # pylint: disable=abstract-method
 
     NAME = "info"
 
-    def run(self, **kwargs):
+    @property
+    def _stack_action(self) -> Any:
+        """Run against a step."""
+        return None
+
+    def run(self, *_args: Any, **_kwargs: Any) -> None:
         """Get information on CloudFormation stacks."""
         LOGGER.info("outputs for stacks: %s", self.context.get_fqn())
-        if not self.context.get_stacks():
+        if not self.context.stacks:
             LOGGER.warning("no stacks detected (error in config?)")
-        for stack in self.context.get_stacks():
-            provider = self.build_provider(stack)
+        for stack in self.context.stacks:
+            provider = self.build_provider()
 
             try:
                 provider_stack = provider.get_stack(stack.fqn)
             except exceptions.StackDoesNotExist:
-                LOGGER.info(
-                    "%s:stack does not exist", stack.fqn,
-                )
+                LOGGER.info("%s:stack does not exist", stack.fqn)
                 continue
 
             LOGGER.info("%s:", stack.fqn)
