@@ -1,15 +1,21 @@
 """Hook utils."""
-import collections
+from __future__ import annotations
+
+import collections.abc
 import logging
 import os
 import sys
-from types import FunctionType
-
-from runway.util import load_object_from_string
-from runway.variables import Variable, resolve_variables
+from typing import TYPE_CHECKING, Any, Dict, List, Mapping, cast
 
+from ...exceptions import FailedVariableLookup
+from ...utils import load_object_from_string
+from ...variables import Variable, resolve_variables
 from ..blueprints.base import Blueprint
-from ..exceptions import FailedVariableLookup
+
+if TYPE_CHECKING:
+    from ...config.models.cfngin import CfnginHookDefinitionModel
+    from ...context import CfnginContext
+    from ..providers.aws.default import Provider
 
 LOGGER = logging.getLogger(__name__)
 
@@ -17,49 +23,47 @@ LOGGER = logging.getLogger(__name__)
 class BlankBlueprint(Blueprint):
     """Blueprint that can be built programatically."""
 
-    def create_template(self):
+    def create_template(self) -> None:
         """Create template without raising NotImplementedError."""
 
 
-def full_path(path):
+def full_path(path: str) -> str:
     """Return full path."""
     return os.path.abspath(os.path.expanduser(path))
 
 
 # TODO split up to reduce number of statements
 def handle_hooks(  # pylint: disable=too-many-statements
-    stage, hooks, provider, context
+    stage: str,
+    hooks: List[CfnginHookDefinitionModel],
+    provider: Provider,
+    context: CfnginContext,
 ):
-    """Handle pre/post_build hooks.
+    """Handle pre/post_deploy hooks.
 
-    These are pieces of code that we want to run before/after the builder
-    builds the stacks.
+    These are pieces of code that we want to run before/after deploying
+    stacks.
 
     Args:
-        stage (str): The current stage (pre_run, post_run, etc).
-        hooks (List[:class:`runway.cfngin.config.Hook`]): Hooks to execute.
-        provider (:class:`runway.cfngin.providers.base.BaseProvider`): Provider
-            instance.
-        context (:class:`runway.cfngin.context.Context`): Context instance.
+        stage: The current stage (pre_run, post_run, etc).
+        hooks: Hooks to execute.
+        provider: Provider instance.
+        context: Context instance.
 
     """
     if not hooks:
         LOGGER.debug("no %s hooks defined", stage)
         return
 
-    hook_paths = []
+    hook_paths: List[str] = []
     for i, hook in enumerate(hooks):
         try:
             hook_paths.append(hook.path)
-        except KeyError:
-            raise ValueError("%s hook #%d missing path." % (stage, i))
+        except KeyError as exc:
+            raise ValueError("%s hook #%d missing path." % (stage, i)) from exc
 
     LOGGER.info("executing %s hooks: %s", stage, ", ".join(hook_paths))
-    stage = stage.replace("build", "deploy")  # TODO remove after full rename
     for hook in hooks:
-        data_key = hook.data_key
-        required = hook.required
-
         if not hook.enabled:
             LOGGER.debug("hook with method %s is disabled; skipping", hook.path)
             continue
@@ -68,13 +72,13 @@ def handle_hooks(  # pylint: disable=too-many-statements
             method = load_object_from_string(hook.path, try_reload=True)
         except (AttributeError, ImportError):
             LOGGER.exception("unable to load method at %s", hook.path)
-            if required:
+            if hook.required:
                 raise
             continue
 
-        if isinstance(hook.args, dict):
+        if hook.args:
             args = [Variable(k, v) for k, v in hook.args.items()]
-            try:  # handling for output or similar being used in pre_build
+            try:  # handling for output or similar being used in pre_deploy
                 resolve_variables(args, context, provider)
             except FailedVariableLookup:
                 if "pre" in stage:
@@ -86,25 +90,25 @@ def handle_hooks(  # pylint: disable=too-many-statements
                         "does not exist yet"
                     )
                 raise
-            kwargs = {v.name: v.value for v in args}
+            kwargs: Dict[str, Any] = {v.name: v.value for v in args}
         else:
-            kwargs = hook.args or {}
+            kwargs = {}
 
         try:
-            if isinstance(method, FunctionType):
-                result = method(context=context, provider=provider, **kwargs)
-            else:
+            if isinstance(method, type):
                 result = getattr(
                     method(context=context, provider=provider, **kwargs), stage
                 )()
+            else:
+                result = method(context=context, provider=provider, **kwargs)
         except Exception:  # pylint: disable=broad-except
-            LOGGER.exception("method %s threw an exception", hook.path)
-            if required:
+            LOGGER.exception("hook %s threw an exception", hook.path)
+            if hook.required:
                 raise
             continue
 
         if not result:
-            if required:
+            if hook.required:
                 LOGGER.error(
                     "required hook %s failed; return value: %s", hook.path, result
                 )
@@ -113,14 +117,16 @@ def handle_hooks(  # pylint: disable=too-many-statements
                 "non-required hook %s failed; return value: %s", hook.path, result
             )
         else:
-            if isinstance(result, collections.Mapping):
-                if data_key:
+            if isinstance(result, collections.abc.Mapping):
+                if hook.data_key:
                     LOGGER.debug(
                         "adding result for hook %s to context in data_key %s",
                         hook.path,
-                        data_key,
+                        hook.data_key,
+                    )
+                    context.set_hook_data(
+                        hook.data_key, cast(Mapping[str, Any], result)
                     )
-                    context.set_hook_data(data_key, result)
                 else:
                     LOGGER.debug(
                         "hook %s returned result data but no data key set; ignoring",
