@@ -1,12 +1,15 @@
 """AWS KMS lookup."""
-# pylint: disable=arguments-differ,unused-argument
+# pyright: reportIncompatibleMethodOverride=none
+from __future__ import annotations
+
 import codecs
-import sys
+from typing import TYPE_CHECKING, Any
 
-from runway.lookups.handlers.base import LookupHandler
+from ....lookups.handlers.base import LookupHandler
+from ...utils import read_value_from_path
 
-from ...session_cache import get_session
-from ...util import read_value_from_path
+if TYPE_CHECKING:
+    from ....context import CfnginContext
 
 TYPE_NAME = "kms"
 
@@ -15,14 +18,14 @@ class KmsLookup(LookupHandler):
     """AWS KMS lookup."""
 
     @classmethod
-    def handle(cls, value, context=None, provider=None, **kwargs):
+    def handle(  # pylint: disable=arguments-differ
+        cls, value: str, context: CfnginContext, **_: Any
+    ) -> str:
         r"""Decrypt the specified value with a master key in KMS.
 
         Args:
-            value (str): Parameter(s) given to this lookup.
-            context (:class:`runway.cfngin.context.Context`): Context instance.
-            provider (:class:`runway.cfngin.providers.base.BaseProvider`):
-                Provider instance.
+            value: Parameter(s) given to this lookup.
+            context: Context instance.
 
         ``value`` should be in the following format:
 
@@ -65,16 +68,13 @@ class KmsLookup(LookupHandler):
         if "@" in value:
             region, value = value.split("@", 1)
 
-        kms = get_session(region).client("kms")
-
-        # encode str value as an utf-8 bytestring for use with codecs.decode.
-        value = value.encode("utf-8")
+        kms = context.get_session(region=region).client("kms")
 
         # get raw but still encrypted value from base64 version.
-        decoded = codecs.decode(value, "base64")
+        decoded = codecs.decode(value.encode(), "base64")
 
         # decrypt and return the plain text raw value.
         decrypted = kms.decrypt(CiphertextBlob=decoded)["Plaintext"]
-        if sys.version_info[0] > 2:  # TODO remove condition after droping py2
+        if isinstance(decrypted, bytes):
             return decrypted.decode()
-        return decrypted
+        return decrypted.read().decode()
