@@ -1,53 +1,38 @@
 """Provider base class."""
-# pylint: disable=no-self-use,too-few-public-methods,unused-argument
+# pylint: disable=no-self-use,unused-argument
+from typing import Any, Optional
 
 
-def not_implemented(method):
+def not_implemented(method: str) -> None:
     """Wrap NotImplimentedError with a formatted message."""
     raise NotImplementedError("Provider does not support '%s' method." % method)
 
 
-class BaseProviderBuilder(object):
+class BaseProviderBuilder:
     """ProviderBuilder base class."""
 
-    def build(self, region=None):
+    def build(self, region: Optional[str] = None) -> Any:
         """Abstract method."""
         not_implemented("build")
 
 
-class BaseProvider(object):
+class BaseProvider:
     """Provider base class."""
 
-    def get_stack(self, stack_name, *args, **kwargs):
+    def get_stack(self, stack_name: str, *args: Any, **kwargs: Any) -> Any:
         """Abstract method."""
         not_implemented("get_stack")
 
-    def create_stack(self, *args, **kwargs):
-        """Abstract method."""
-        not_implemented("create_stack")
-
-    def update_stack(self, *args, **kwargs):
-        """Abstract method."""
-        not_implemented("update_stack")
-
-    def destroy_stack(self, stack, *args, **kwargs):
-        """Abstract method."""
-        not_implemented("destroy_stack")
-
-    def get_stack_status(self, stack, *args, **kwargs):
-        """Abstract method."""
-        not_implemented("get_stack_status")
-
-    def get_outputs(self, stack_name, *args, **kwargs):
+    def get_outputs(self, stack_name: str, *args: Any, **kwargs: Any) -> Any:
         """Abstract method."""
         not_implemented("get_outputs")
 
-    def get_output(self, stack, output):
+    def get_output(self, stack: str, output: str) -> str:
         """Abstract method."""
         return self.get_outputs(stack)[output]
 
 
-class Template(object):
+class Template:
     """CloudFormation stack template, which could be optionally uploaded to s3.
 
     Presence of the url attribute indicates that the template was uploaded to
@@ -56,7 +41,7 @@ class Template(object):
 
     """
 
-    def __init__(self, url=None, body=None):
+    def __init__(self, url: Optional[str] = None, body: Optional[str] = None) -> None:
         """Instantiate class."""
         self.url = url
         self.body = body
