@@ -1,26 +1,32 @@
 """Kubectl version management."""
+from __future__ import annotations
+
 import logging
 import os
 import platform
 import shutil
 import sys
 import tempfile
+from typing import TYPE_CHECKING, Optional, cast
+from urllib.error import URLError
+from urllib.request import urlretrieve
 
 import requests
 
-# Old pylint on py2.7 incorrectly flags these
-from six.moves.urllib.error import URLError  # pylint: disable=E
-from six.moves.urllib.request import urlretrieve  # pylint: disable=E
-
-from ..util import get_file_hash
+from ..utils import get_file_hash
 from . import EnvManager, handle_bin_download_error
 
-LOGGER = logging.getLogger(__name__)
+if TYPE_CHECKING:
+    from pathlib import Path
+
+    from .._logging import RunwayLogger
+
+LOGGER = cast("RunwayLogger", logging.getLogger(__name__))
 KB_VERSION_FILENAME = ".kubectl-version"
 RELEASE_URI = "https://storage.googleapis.com/kubernetes-release/release"
 
 
-def verify_kb_release(kb_url, download_dir, filename):
+def verify_kb_release(kb_url: str, download_dir: str, filename: str) -> None:
     """Compare checksum and exit if it doesn't match.
 
     Different releases provide varying checksum files. To account for this,
@@ -96,13 +102,16 @@ def verify_kb_release(kb_url, download_dir, filename):
 
 
 def download_kb_release(
-    version, versions_dir, kb_platform=None, arch=None,
-):
+    version: str,
+    versions_dir: Path,
+    kb_platform: Optional[str] = None,
+    arch: Optional[str] = None,
+) -> None:
     """Download kubectl and return path to it."""
     version_dir = versions_dir / version
 
     if arch is None:
-        arch = os.environ.get("KBENV_ARCH") if os.environ.get("KBENV_ARCH") else "amd64"
+        arch = os.getenv("KBENV_ARCH", "amd64")
 
     if not kb_platform:
         if platform.system().startswith("Darwin"):
@@ -125,20 +134,19 @@ def download_kb_release(
     try:
         LOGGER.verbose("downloading kubectl from %s...", kb_url)
         urlretrieve(kb_url + "/" + filename, os.path.join(download_dir, filename))
-    # IOError in py2; URLError in 3+
-    except (IOError, URLError) as exc:
+    except URLError as exc:
         handle_bin_download_error(exc, "kubectl")
 
     verify_kb_release(kb_url, download_dir, filename)
 
     version_dir.mkdir(parents=True, exist_ok=True)
-    shutil.move(os.path.join(download_dir, filename), str(version_dir / filename))
+    shutil.move(os.path.join(download_dir, filename), version_dir / filename)
     shutil.rmtree(download_dir)
     result = version_dir / filename
     result.chmod(result.stat().st_mode | 0o0111)  # ensure it is executable
 
 
-def get_version_requested(path):
+def get_version_requested(path: Path) -> str:
     """Return string listing requested kubectl version."""
     kb_version_path = path / KB_VERSION_FILENAME
     if not kb_version_path.is_file():
@@ -152,18 +160,18 @@ def get_version_requested(path):
     return kb_version_path.read_text().strip()
 
 
-class KBEnvManager(EnvManager):  # pylint: disable=too-few-public-methods
+class KBEnvManager(EnvManager):
     """kubectl version management.
 
     Designed to be compatible with https://github.com/alexppg/kbenv.
 
     """
 
-    def __init__(self, path=None):
+    def __init__(self, path: Optional[Path] = None) -> None:
         """Initialize class."""
-        super(KBEnvManager, self).__init__("kubectl", "kbenv", path)
+        super().__init__("kubectl", "kbenv", path)
 
-    def install(self, version_requested=None):
+    def install(self, version_requested: Optional[str] = None) -> str:
         """Ensure kubectl is available."""
         if not version_requested:
             version_requested = get_version_requested(self.path)
@@ -175,7 +183,7 @@ class KBEnvManager(EnvManager):  # pylint: disable=too-few-public-methods
         # matching version is already installed
         if (self.versions_dir / version_requested).is_dir():
             LOGGER.verbose(
-                "kubectl version %s already installed; using it...", version_requested,
+                "kubectl version %s already installed; using it...", version_requested
             )
             self.current_version = version_requested
             return str(self.bin)
